﻿#region File Description
//-----------------------------------------------------------------------------
// HighscoreScreen.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

#region Using Statements
using System;
using System.IO;
using System.Diagnostics;
using System.Xml.Serialization;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace TetrisDreams
{
    public class CreditsPost
    {
    public String text;
    public Vector2 pos;
    public Color color;
    public float size;

    public CreditsPost(String txt, Vector2 position, Color col, float sz)
        {
        text = txt;
        pos = position;
        color = col;
        size = sz;
        }
    }



    /// <summary>
    /// The Credits screen displays the involved developers in a classic
    /// credits list.
    /// </summary>
    class CreditsScreen : GameScreen
    {
        #region Fields

        ContentManager content;

        float alpha = 0;
        Texture2D textureAlphaMask;
        int pos = 0;

        int screenHeight;
        int screenWidth;

        CreditsPost[] creditsPost = new CreditsPost[23];

        #endregion

        #region Initialization


        /// <summary>
        /// Constructor.
        /// </summary>
        public CreditsScreen()
            {                  
            }


        /// <summary>
        /// Loads graphics content for this screen.
        /// </summary>
        public override void LoadContent()
            {
            if (content == null)
                content = new ContentManager(ScreenManager.Game.Services, "Content");

            screenWidth = ScreenManager.GraphicsDevice.Viewport.Width;
            screenHeight = ScreenManager.GraphicsDevice.Viewport.Height;

            pos = screenHeight;

            int creditPosX = screenWidth/2 - 400;

            creditsPost[0] = new CreditsPost("[BITYARD]", new Vector2(creditPosX, 0), Color.White, 1f);
            creditsPost[1] = new CreditsPost("Coding", new Vector2(creditPosX, 80), Color.DarkGray, 1f);
            creditsPost[2] = new CreditsPost("laumsporre", new Vector2(creditPosX, 120), Color.White, 0.8f);
            creditsPost[3] = new CreditsPost("Testing", new Vector2(creditPosX, 200), Color.DarkGray, 1f);
            creditsPost[4] = new CreditsPost("dee", new Vector2(creditPosX, 240), Color.White, 0.8f);
            creditsPost[5] = new CreditsPost("Dimorgh", new Vector2(creditPosX, 280), Color.White, 0.8f);
            creditsPost[6] = new CreditsPost("Music", new Vector2(creditPosX, 360), Color.DarkGray, 1f);
            creditsPost[7] = new CreditsPost("Maciej Jasiszczak", new Vector2(creditPosX, 400), Color.White, 0.8f);
            creditsPost[8] = new CreditsPost("Tetris in dreams", new Vector2(creditPosX, 440), Color.White, 0.8f);
            creditsPost[9] = new CreditsPost("remix.kwed.org", new Vector2(creditPosX, 480), Color.White, 0.8f);
            creditsPost[10] = new CreditsPost("House Mascot", new Vector2(creditPosX, 560), Color.DarkGray, 1f);
            creditsPost[11] = new CreditsPost("thoulouse", new Vector2(creditPosX, 600), Color.White, 0.8f);
            creditsPost[12] = new CreditsPost("Publisher", new Vector2(creditPosX, 680), Color.DarkGray, 1f);
            creditsPost[13] = new CreditsPost("CSharpSkolan.se", new Vector2(creditPosX, 720), Color.White, 0.8f);
            creditsPost[14] = new CreditsPost("Thanks", new Vector2(creditPosX, 800), Color.DarkGray, 1f);
            creditsPost[15] = new CreditsPost("David Amador - independent resolution rendering", new Vector2(creditPosX, 840), Color.White, 0.8f);
            creditsPost[16] = new CreditsPost("msdn.microsoft.com", new Vector2(creditPosX, 880), Color.White, 0.8f);
            creditsPost[17] = new CreditsPost("Asus ProArt PA246Q", new Vector2(creditPosX, 920), Color.White, 0.8f);
            creditsPost[18] = new CreditsPost("Razor BlackWidow Ultimate", new Vector2(creditPosX, 960), Color.White, 0.8f);
            creditsPost[19] = new CreditsPost("Caffeine", new Vector2(creditPosX, 1000), Color.White, 0.8f);
            creditsPost[20] = new CreditsPost("Copyright", new Vector2(creditPosX, 1080), Color.DarkGray, 1f);
            creditsPost[21] = new CreditsPost("None :)", new Vector2(creditPosX, 1120), Color.White, 0.8f);
            creditsPost[22] = new CreditsPost("Source code available at CSharpSkolan.se", new Vector2(creditPosX, 1160), Color.White, 0.8f);

            textureAlphaMask = new Texture2D(ScreenManager.GraphicsDevice, screenWidth, screenHeight);

            Color[] color = new Color[screenWidth * screenHeight];
            
            byte dim = 255;

            // Create an alpha gradient
            for (int i = 0; i < color.Length; i++)
                {
                color[i] = new Color(0,0,0);

                if (i % screenWidth == 0 && i > screenWidth * 100 & i < screenWidth * 355 && dim > 2)
                    dim -= 2;
                else
                    if (i % screenWidth == 0 && i > screenWidth * (screenHeight - 228) && dim < 254)
                    dim += 2;
                
                color[i].A = dim;
                }

            // Create the alpha gradient texture from our colors
            textureAlphaMask.SetData(color, 0, screenWidth * screenHeight);
            }


        /// <summary>
        /// Unloads graphics content for this screen.
        /// </summary>
        public override void UnloadContent()
            {
            content.Unload();
            }


        #endregion

        #region Update and Draw


        /// <summary>
        /// Updates the background screen. Unlike most screens, this should not
        /// transition off even if it has been covered by another screen: it is
        /// supposed to be covered, after all! This overload forces the
        /// coveredByOtherScreen parameter to false in order to stop the base
        /// Update method wanting to transition off.
        /// </summary>
        public override void Update(GameTime gameTime, bool otherScreenHasFocus,
                                                       bool coveredByOtherScreen)
            {
            Viewport viewport = ScreenManager.GraphicsDevice.Viewport;

            if(alpha < 1)
                alpha += 0.02f;

            base.Update(gameTime, otherScreenHasFocus, false);

            pos -= 1;
            if( creditsPost[creditsPost.Length-1].pos.Y + pos < 0 )
                pos = screenHeight;       
            }

        public override void HandleInput(InputState input)
            {
            if (input == null)
                throw new ArgumentNullException("input");

            // Look up inputs for the active player profile.
            PlayerIndex playerIndex = ControllingPlayer.Value;

            KeyboardState keyboardState = input.CurrentKeyboardStates[(int)playerIndex];
            GamePadState gamePadState = input.CurrentGamePadStates[(int)playerIndex];

            // The game pauses either if the user presses the pause button, or if
            // they unplug the active gamepad. This requires us to keep track of
            // whether a gamepad was ever plugged in, because we don't want to pause
            // on PC if they are playing with a keyboard and have no gamepad at all!
            bool gamePadDisconnected = !gamePadState.IsConnected &&
                                       input.GamePadWasConnected[(int)playerIndex];

            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                {
                ScreenManager.RemoveScreen(this);
                ScreenManager.AddScreen(new MainMenuScreen(), null);
                }
            if (input.IsPauseGame(ControllingPlayer) || gamePadDisconnected)
                {
                ScreenManager.RemoveScreen(this);
                ScreenManager.AddScreen(new MainMenuScreen(), null);
                }
            else
                {
                if (input.IsNewKeyPress(Keys.Escape, ControllingPlayer, out playerIndex))
                    {
                    ScreenManager.RemoveScreen(this);
                    ScreenManager.AddScreen(new MainMenuScreen(), null);
                    }
                }
            }

        /// <summary>
        /// Draws the background screen.
        /// </summary>
        public override void Draw(GameTime gameTime)
            {
            SpriteBatch spriteBatch = ScreenManager.SpriteBatch;
            Viewport viewport = ScreenManager.GraphicsDevice.Viewport;

            // Dim the background a little bit
            ScreenManager.FadeBackBufferToBlack(alpha * 2 / 3);

            SpriteFont font = ScreenManager.Font;
            Vector2 origin = new Vector2(0, font.LineSpacing / 2);

            spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend, null, null, null, null, Resolution.getTransformationMatrix());

            // Draw credit text
            for(int i=0; i<creditsPost.Length; i++)
                {
                CreditsPost post = creditsPost[i];
                spriteBatch.DrawString(font, post.text, post.pos + new Vector2(0, pos), post.color, 0, origin, post.size, SpriteEffects.None, 0);
                }
        
            // Draw alpha mask
            spriteBatch.Draw(textureAlphaMask, new Vector2(0, 0), null, Color.White, 0, new Vector2(0, 0), 1, SpriteEffects.None, 0);
    
            spriteBatch.End();      
            }


        #endregion
    }
}
