#region File Description
//-----------------------------------------------------------------------------
// GameBoardScreen.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

#region Using Statements
using System;
//using System.Collections;
using System.Linq;
//using System.Text;
using System.Threading;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Audio;
#endregion

namespace TetrisDreams
{
    /// <summary>
    /// This screen implements the actual game logic.
    /// </summary>
    class GameBoardScreen : GameScreen
        {
        #region Fields
        
        ContentManager content;
        SpriteFont gameFont;

        Random random = new Random();

        float pauseAlpha;

        Texture2D gameBoard;
        Texture2D gameBoardUpperEdge;
        Texture2D gameWindowLarge;
        Texture2D gameWindowSmall;

        AudioEngine audioEngine;
        WaveBank waveBank;
        static SoundBank soundBank;
        AudioCategory audioCategory;

        public static Texture2D textureBrickBlue;
        public static Texture2D textureBrickGreen;
        public static Texture2D textureBrickOrange;
        public static Texture2D textureBrickPurple;
        public static Texture2D textureBrickRed;
        public static Texture2D textureBrickYellow;
        public static Texture2D textureBrickCyan;

        static Texture2D textureBrickBorder1;
        static Texture2D textureBrickBorder2;
        static Texture2D textureBrickBorder3;
        static Texture2D textureBrickBorder4;
        static Texture2D textureBrickBorder5;
        static Texture2D textureBrickBorder6;
        static Texture2D textureBrickBorder7;
        static Texture2D textureBrickBorder8;
        static Texture2D textureBrickBorder9;
        static Texture2D textureBrickBorder10;
        static Texture2D textureBrickBorder11;
        static Texture2D textureBrickBorder12;
        static Texture2D textureBrickBorder13;
        static Texture2D textureBrickBorder14;
        static Texture2D textureBrickBorder15;
        static Texture2D textureBrickBorder16;
        static Texture2D textureBrickBorder17;
        static Texture2D textureBrickBorder18;


        public Array shadowBlocks { get; private set; }

       
        Vector2 pos = new Vector2(0, 0);

        #endregion

        #region Initialization


        /// <summary>
        /// Constructor.
        /// </summary>
        public GameBoardScreen()
            {
            TransitionOnTime = TimeSpan.FromSeconds(1.5);
            TransitionOffTime = TimeSpan.FromSeconds(0.5);

            //newFallingBlock = Array.CreateInstance(typeof(int), 4, 4);
            shadowBlocks = Array.CreateInstance(typeof(Texture2D), 18);

            TetrisModel.Instance.NewGame();
            TetrisSaveData saveData = new TetrisSaveData();  
            }


        /// <summary>
        /// Load graphics content for the game.
        /// </summary>
        public override void LoadContent()
            {
            if (content == null)
                content = new ContentManager(ScreenManager.Game.Services, "Content");

            gameFont = content.Load<SpriteFont>("fontTexture");
            gameBoard = content.Load<Texture2D>("gameboard");
            gameWindowLarge = content.Load<Texture2D>("gamewindow_large");
            gameWindowSmall = content.Load<Texture2D>("gamewindow_small");

            gameBoardUpperEdge = content.Load<Texture2D>("gameboard_upper_edge");
            textureBrickBlue = content.Load<Texture2D>("cube_1x1_blue");
            textureBrickGreen = content.Load<Texture2D>("cube_1x1_green");
            textureBrickOrange = content.Load<Texture2D>("cube_1x1_orange");
            textureBrickPurple = content.Load<Texture2D>("cube_1x1_purple");
            textureBrickRed = content.Load<Texture2D>("cube_1x1_red");
            textureBrickYellow = content.Load<Texture2D>("cube_1x1_yellow");
            textureBrickCyan = content.Load<Texture2D>("cube_1x1_cyan");

            textureBrickBorder1 = content.Load<Texture2D>("cube_1x1_border_1");
            textureBrickBorder2 = content.Load<Texture2D>("cube_1x1_border_2");
            textureBrickBorder3 = content.Load<Texture2D>("cube_1x1_border_3");
            textureBrickBorder4 = content.Load<Texture2D>("cube_1x1_border_4");
            textureBrickBorder5 = content.Load<Texture2D>("cube_1x1_border_5");
            textureBrickBorder6 = content.Load<Texture2D>("cube_1x1_border_6");
            textureBrickBorder7 = content.Load<Texture2D>("cube_1x1_border_7");
            textureBrickBorder8 = content.Load<Texture2D>("cube_1x1_border_8");
            textureBrickBorder9 = content.Load<Texture2D>("cube_1x1_border_9");
            textureBrickBorder10 = content.Load<Texture2D>("cube_1x1_border_10");
            textureBrickBorder11 = content.Load<Texture2D>("cube_1x1_border_11");
            textureBrickBorder12 = content.Load<Texture2D>("cube_1x1_border_12");
            textureBrickBorder13 = content.Load<Texture2D>("cube_1x1_border_13");
            textureBrickBorder14 = content.Load<Texture2D>("cube_1x1_border_14");
            textureBrickBorder15 = content.Load<Texture2D>("cube_1x1_border_15");
            textureBrickBorder16 = content.Load<Texture2D>("cube_1x1_border_16");
            textureBrickBorder17 = content.Load<Texture2D>("cube_1x1_border_17");
            textureBrickBorder18 = content.Load<Texture2D>("cube_1x1_border_18");

            //backgroundTexture = content.Load<Texture2D>("background");

            shadowBlocks.SetValue(textureBrickBorder1, 0);
            shadowBlocks.SetValue(textureBrickBorder2, 1);
            shadowBlocks.SetValue(textureBrickBorder3, 2);
            shadowBlocks.SetValue(textureBrickBorder4, 3);
            shadowBlocks.SetValue(textureBrickBorder5, 4);
            shadowBlocks.SetValue(textureBrickBorder6, 5);
            shadowBlocks.SetValue(textureBrickBorder7, 6);
            shadowBlocks.SetValue(textureBrickBorder8, 7);
            shadowBlocks.SetValue(textureBrickBorder9, 8);
            shadowBlocks.SetValue(textureBrickBorder10, 9);
            shadowBlocks.SetValue(textureBrickBorder11, 10);
            shadowBlocks.SetValue(textureBrickBorder12, 11);
            shadowBlocks.SetValue(textureBrickBorder13, 12);
            shadowBlocks.SetValue(textureBrickBorder14, 13);
            shadowBlocks.SetValue(textureBrickBorder15, 14);
            shadowBlocks.SetValue(textureBrickBorder16, 15);
            shadowBlocks.SetValue(textureBrickBorder17, 16);
            shadowBlocks.SetValue(textureBrickBorder18, 17);

            audioEngine = new AudioEngine("Content\\sounds.xgs");
            waveBank = new WaveBank(audioEngine, "Content\\Wave Bank.xwb");
            soundBank = new SoundBank(audioEngine, "Content\\Sound Bank.xsb");
            audioCategory = audioEngine.GetCategory("Sfx");

            if( TetrisModel.Instance.tetrisSaveData.sfx )
                {
                audioCategory.SetVolume((float)TetrisModel.Instance.tetrisSaveData.sfxvolyme / 100);
                }
  
            // A real game would probably have more content than this sample, so
            // it would take longer to load. We simulate that by delaying for a
            // while, giving you a chance to admire the beautiful loading screen.
            Thread.Sleep(500);

            // once the load has finished, we use ResetElapsedTime to tell the game's
            // timing mechanism that we have just finished a very long frame, and that
            // it should not try to catch up.
            ScreenManager.Game.ResetElapsedTime();
            }


        /// <summary>
        /// Unload graphics content used by the game.
        /// </summary>
        public override void UnloadContent()
            {
            content.Unload();
            }


        #endregion   

        #region Misc

        public static void PlaySfx()
            {
            soundBank.PlayCue("sfx_clearline");
            }

        #endregion

        #region Update and Draw


        /// <summary>
        /// Updates the state of the game. This method checks the GameScreen.IsActive
        /// property, so the game will stop updating when the pause menu is active,
        /// or if you tab away to a different application.
        /// </summary>
        public override void Update(GameTime gameTime, bool otherScreenHasFocus,
                                                       bool coveredByOtherScreen)
            {
            base.Update(gameTime, otherScreenHasFocus, false);

            // Gradually fade in or out depending on whether we are covered by the pause screen.
            if (coveredByOtherScreen)
                pauseAlpha = Math.Min(pauseAlpha + 1f / 32, 1);
            else
                pauseAlpha = Math.Max(pauseAlpha - 1f / 32, 0);

            if (IsActive)
                {
                if( TetrisModel.Instance.DoUpdate(gameTime) == false )
                    {
                    ScreenManager.RemoveScreen(this);
                    ScreenManager.AddScreen(new BackgroundScreen(), null);
                    ScreenManager.AddScreen(new MainMenuScreen(), null);
                    ScreenManager.AddScreen(new HighscoreScreen(), PlayerIndex.One);
                    }
                }

            // Update the audio engine.
            audioEngine.Update();
        }


        /// <summary>
        /// Lets the game respond to player input. Unlike the Update method,
        /// this will only be called when the gameplay screen is active.
        /// </summary>
        public override void HandleInput(InputState input)
            {
            if (input == null)
                throw new ArgumentNullException("input");

            // Look up inputs for the active player profile.
            PlayerIndex playerIndex = ControllingPlayer.Value;

            KeyboardState keyboardState = input.CurrentKeyboardStates[(int)playerIndex];
            GamePadState gamePadState = input.CurrentGamePadStates[(int)playerIndex];

            // The game pauses either if the user presses the pause button, or if
            // they unplug the active gamepad. This requires us to keep track of
            // whether a gamepad was ever plugged in, because we don't want to pause
            // on PC if they are playing with a keyboard and have no gamepad at all!
            bool gamePadDisconnected = !gamePadState.IsConnected &&
                                       input.GamePadWasConnected[(int)playerIndex];

            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                {
                }
                        
            if (input.IsPauseGame(ControllingPlayer) || gamePadDisconnected)
                {
                ScreenManager.AddScreen(new PauseMenuScreen(), ControllingPlayer);
                }
            else
                {
                // XBox 360
                if (input.IsNewButtonPress(Buttons.DPadLeft, ControllingPlayer, out playerIndex) ||
                    input.IsLongButtonPress(Buttons.DPadLeft, ControllingPlayer, out playerIndex))
                    {
                    TetrisModel.Instance.MoveBlock(-1);
                    }
                if (input.IsNewButtonPress(Buttons.DPadRight, ControllingPlayer, out playerIndex) ||
                    input.IsLongButtonPress(Buttons.DPadRight, ControllingPlayer, out playerIndex))
                    {
                    TetrisModel.Instance.MoveBlock(1);
                    }

                if (input.IsNewButtonPress(Buttons.DPadUp, ControllingPlayer, out playerIndex) ||
                    input.IsLongButtonPress(Buttons.DPadUp, ControllingPlayer, out playerIndex))
                    {
                    TetrisModel.Instance.RotateBlock(true);
                    }

                if (input.IsNewButtonPress(Buttons.DPadDown, ControllingPlayer, out playerIndex))
                    {
                    TetrisModel.Instance.DropBlock();
                    }
                if (input.IsLongButtonPress(Buttons.DPadDown, ControllingPlayer, out playerIndex))
                    {
                    TetrisModel.Instance.DropBlockAgain();
                    }

                // Windows
                if (input.IsNewKeyPress(Keys.Left, ControllingPlayer, out playerIndex) ||
                    input.IsLongKeyPress(Keys.Left, ControllingPlayer, out playerIndex))
                    {
                    TetrisModel.Instance.MoveBlock(-1);
                    }

                if (input.IsNewKeyPress(Keys.Right, ControllingPlayer, out playerIndex) ||
                    input.IsLongKeyPress(Keys.Right, ControllingPlayer, out playerIndex))
                    {
                    TetrisModel.Instance.MoveBlock(1);
                    }

                if (input.IsNewKeyPress(Keys.Up, ControllingPlayer, out playerIndex) ||
                    input.IsLongKeyPress(Keys.Up, ControllingPlayer, out playerIndex))
                    {
                    TetrisModel.Instance.RotateBlock(true);
                    }

                if (input.IsNewKeyPress(Keys.Down, ControllingPlayer, out playerIndex))
                    {
                    TetrisModel.Instance.DropBlock();
                    }
                if (input.IsLongKeyPress(Keys.Down, ControllingPlayer, out playerIndex))
                    {
                    TetrisModel.Instance.DropBlockAgain();
                    }
                if (input.IsNewKeyPress(Keys.Q, ControllingPlayer, out playerIndex))
                    {
                    TetrisModel.Instance.gameState = EGameState.eGameOver;
                    }    
                }
            }


        /// <summary>
        /// Draws the gameplay screen.
        /// </summary>
        public override void Draw(GameTime gameTime)
            {
            ScreenManager.GraphicsDevice.Clear(ClearOptions.Target,
                                               Color.Black, 0, 0);


            SpriteBatch spriteBatch = ScreenManager.SpriteBatch;
            Viewport viewport = ScreenManager.GraphicsDevice.Viewport;

            int height = viewport.Height;
            int width = viewport.Width;

            int middlescreenX = 1920/2;

            Rectangle fullscreen = new Rectangle(0, 0, viewport.Width, viewport.Height);
            
            
            // Begin drawing
            Resolution.BeginDraw();
            spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend, null, null, null, null, Resolution.getTransformationMatrix());

            spriteBatch.Draw(gameBoard, new Vector2(middlescreenX - gameBoard.Width / 2, 100), Color.White * 0.5f);
            spriteBatch.Draw(gameWindowLarge, new Vector2(middlescreenX - gameBoard.Width / 2 - 40 - gameWindowLarge.Width, 150), Color.White);
            spriteBatch.Draw(gameWindowLarge, new Vector2(middlescreenX + gameBoard.Width/2 + 40, 150), Color.White);
            spriteBatch.Draw(gameWindowSmall, new Vector2(middlescreenX - gameBoard.Width / 2 - 40 - gameWindowLarge.Width, 730), Color.White);
            spriteBatch.Draw(gameWindowSmall, new Vector2(middlescreenX - gameBoard.Width / 2 - 40 - gameWindowLarge.Width, 880), Color.White);
            spriteBatch.Draw(gameWindowSmall, new Vector2(middlescreenX + gameBoard.Width / 2 + 40, 730), Color.White);
            spriteBatch.Draw(gameWindowSmall, new Vector2(middlescreenX + gameBoard.Width / 2 + 40, 880), Color.White);         

            Vector2 offset = new Vector2(259, 84);

            switch (TetrisModel.Instance.gameState)
                {
                case EGameState.eBlockInPlay:
                case EGameState.eWaitingForBlocks:
                case EGameState.eAddNewBlock:
                    {
                    if (TetrisModel.Instance.gameState == EGameState.eBlockInPlay)
                        {
                        for (int i = 0; i < 4; i++)
                            {
                            for (int n = 0; n < 4; n++)
                                {
                                TetrisModel.Instance.tetrisBlockQueue.NextBlock(ref TetrisModel.Instance.aBlock);
                                int blockvalue = (int)TetrisModel.Instance.aBlock.blockArray.GetValue(i, n);
                                if ( blockvalue != 0)
                                    {
                                    pos.X = n * 40 + (TetrisModel.Instance.aBlock.posX - 3) * 40 + 80 + 539;
                                    pos.Y = (i + 1) * 40 + TetrisModel.Instance.aBlock.posY * 40;

                                    //Draw block
                                    spriteBatch.Draw(TetrisModel.Instance.aBlock.GetTexture(), pos + offset, Color.White);
                                    //Draw shadow
                                    pos.Y += (TetrisModel.Instance.blockShadowPos - 1) * 40;
                                    spriteBatch.Draw((Texture2D)shadowBlocks.GetValue(blockvalue - 1), pos + offset, TetrisModel.Instance.aBlock.GetShadowRGB());          
                                    }
                                }
                            }
                        }

                        
                    int count = 0;
                    offset.X = 675;
                    offset.Y = 0;

                    Vector2 originn = new Vector2(0,0);

                    foreach (Object o in TetrisModel.Instance.tetrisBlockQueue.myQueue)
                        {
                        if (count == 0)
                            {
                            count++;
                            continue;
                            }

                        count++;
                            
                        TetrisBlock block = (TetrisBlock)o;
                            
                        for (int i = 0; i < 4; i++)
                            {
                            for (int n = 0; n < 4; n++)
                                {
                                if ((int)block.blockArray.GetValue(i, n) == 0)
                                    continue;

                                Vector2 queuePos = block.GetQueuePosition();
                                if(count == 2 )
                                    {
                                    
                                    pos.X = (n+queuePos.X)* 40 + 539;
                                    pos.Y = (i + queuePos.Y) * 40 + (count - 1) * 120;
                                    spriteBatch.Draw(block.GetTexture(), pos + offset, Color.White);
                                    }
                                else
                                    {

                                    pos.X = (n + queuePos.X) * 32 + 539 + offset.X;
                                    pos.Y = (i + queuePos.Y) * 32 + (count - 1) * 100 + 70 + offset.Y;
                                    spriteBatch.Draw(block.GetTexture(), pos, null, Color.White, 0.0f, originn, 0.8f, SpriteEffects.None, 0);
                                    }
                                }
                            }    
                        }
                        
                    offset.X = 739;
                    offset.Y = 84;

                    for(int i = 0; i < 20; i++)
                        {
                        for(int n = 0; n < 10; n++)
                            {
                            pos.X = n * 40 + 19;
                            pos.Y = (i - 1) * 40 + 80;
              
                            EBlockColor bcolor = TetrisModel.Instance.gameBoardMatrix[i, n];
                            if (bcolor != EBlockColor.eNone)
                                spriteBatch.Draw(TetrisBlock.GetTexture(bcolor), pos + offset, Color.White);
                            }
                        }
                    

                    }
                    break;
                }

            pos.X = 800;
            pos.Y = 300;
           
            spriteBatch.Draw(gameBoardUpperEdge, new Vector2(739, 16), Color.White);

            pos.X = 1212;
            pos.Y = 130;

            SpriteFont font = ScreenManager.Font;
            Vector2 origin = new Vector2(0, font.LineSpacing / 2);
            Color color = new Color(181, 196, 199);

            spriteBatch.DrawString(font, "HOLD", new Vector2(middlescreenX - gameBoard.Width / 2 - 40 - gameWindowLarge.Width, 130), color, 0, origin, 1, SpriteEffects.None, 0);
            spriteBatch.DrawString(font, "LEVEL", new Vector2(middlescreenX - gameBoard.Width / 2 - 40 - gameWindowLarge.Width, 710), color, 0, origin, 1, SpriteEffects.None, 0);
            spriteBatch.DrawString(font, "GOAL", new Vector2(middlescreenX - gameBoard.Width / 2 - 40 - gameWindowLarge.Width, 860), color, 0, origin, 1, SpriteEffects.None, 0);
            spriteBatch.DrawString(font, "NEXT", new Vector2(middlescreenX + gameBoard.Width / 2 + 40, 130), color, 0, origin, 1, SpriteEffects.None, 0);
            spriteBatch.DrawString(font, "NEXT BEST", new Vector2(middlescreenX + gameBoard.Width / 2 + 40, 710), color, 0, origin, 1, SpriteEffects.None, 0);
            spriteBatch.DrawString(font, "SCORE", new Vector2(middlescreenX + gameBoard.Width / 2 + 40, 860), color, 0, origin, 1, SpriteEffects.None, 0);
            
            spriteBatch.DrawString(font, TetrisModel.Instance.tetrisScore.ToString(), new Vector2(middlescreenX + gameBoard.Width / 2 + 80, 912), color, 0, origin, 1, SpriteEffects.None, 0);
            spriteBatch.DrawString(font, TetrisModel.Instance.tetrisLevel.ToString(), new Vector2(middlescreenX - gameBoard.Width / 2 - 150, 762), color, 0, origin, 1, SpriteEffects.None, 0);


            // End drawing
            spriteBatch.End();


            // If the game is transitioning on or off, fade it out to black.
            if (TransitionPosition > 0 || pauseAlpha > 0)
                {
                float alpha = MathHelper.Lerp(1f - TransitionAlpha, 1f, pauseAlpha / 2);
                ScreenManager.FadeBackBufferToBlack(alpha);
                }

            }

        #endregion
        } // GameBoardScreen
}
