﻿#region File Description

#endregion

#region Using Statements
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
#endregion

namespace TetrisDreams
    {
    public enum EBlockColor
        {
        eNone = 0,
        eYellow = 1,
        eCyan = 2,
        eGreen = 3,
        eRed = 4,
        eOrange = 5,
        eBlue = 6,
        ePurple = 7,
        };

    
    public class TetrisBlock
    {
        #region Fields

        public static readonly Color[] tetrisColors = { Color.Yellow, Color.Cyan, Color.Green, Color.Red, Color.Orange, Color.Blue, Color.Purple };

        // *** BLOCK 0 ***//
        public static readonly int[,] BlockO =  {
                                     {0, 0, 0, 0},
                                     {0, 5, 6, 0},
                                     {0, 9, 10,0},
                                     {0, 0, 0, 0}
                                  };

        // *** BLOCK I ***//
        public static readonly int[,] BlockI =  {                 
                                     {0, 0, 0, 0},
                                     {7, 1, 1, 8},
                                     {0, 0, 0, 0},
                                     {0, 0, 0, 0}
                                  };
        public static readonly int[,] BlockI2 = {
                                     {0, 0, 3, 0},
                                     {0, 0, 2, 0},
                                     {0, 0, 2, 0},
                                     {0, 0, 4, 0}
                                  };

        // *** BLOCK S ***//
        public static readonly int[,] BlockS = {
                                     {0, 0, 0, 0},
                                     {0, 0, 15,8},
                                     {0, 7, 18,0},
                                     {0, 0, 0, 0}
                                   };
        public static readonly int[,] BlockS2 = {
                                     {0,0,3,0},
                                     {0,0,17,16},
                                     {0,0,0,4},
                                     {0,0,0,0}
                                   };

        // *** BLOCK Z ***//
        public static readonly int[,] BlockZ = {
                                     {0,0,0,0},
                                     {0,7,16,0},
                                     {0,0,17,8},
                                     {0,0,0,0}
                                  };
        public static readonly int[,] BlockZ2 = {
                                     {0, 0, 0, 3},
                                     {0, 0, 15,18},
                                     {0, 0, 4, 0},
                                     {0, 0, 0, 0}
                                  };

        // *** BLOCK L ***//
        public static readonly int[,] BlockL1 = {
                                     {0,0,0,3},
                                     {0,7,1,18},
                                     {0,0,0,0},
                                     {0,0,0,0}
                                     
                                  };
        public static readonly int[,] BlockL2 = {
                                     {0,0,3,0},
                                     {0,0,2,0},
                                     {0,0,17,8},
                                     {0,0,0,0}
                                  };
        public static readonly int[,] BlockL3 = {
                                     {0,0,0,0},
                                     {0,15,1,8},
                                     {0,4,0,0},
                                     {0,0,0,0}
                                  };
        public static readonly int[,] BlockL4 = {
                                     {0,7,16,0},
                                     {0,0,2,0},
                                     {0,0,4,0},
                                     {0,0,0,0}
                                  };

        // *** BLOCK J ***//
        public static readonly int[,] BlockJ1 = {
                                     {0,3,0,0},
                                     {0,17,1,8},
                                     {0,0,0,0},
                                     {0,0,0,0} 
                                  };
        public static readonly int[,] BlockJ2 = {
                                     {0,0,15,8},
                                     {0,0,2,0},
                                     {0,0,4,0},
                                     {0,0,0,0}
                                  };
        public static readonly int[,] BlockJ3 = {
                                     {0,0,0,0},
                                     {0,7,1,16},
                                     {0,0,0,4},
                                     {0,0,0,0}
                                  };
        public static readonly int[,] BlockJ4 = {
                                     {0,0,3,0},
                                     {0,0,2,0},
                                     {0,7,18,0},
                                     {0,0,0,0}
                                  };

        // *** BLOCK T ***//
        public static readonly int[,] BlockT1 = {
                                     {0,0,3,0},
                                     {0,7,11,8},
                                     {0,0,0,0},
                                     {0,0,0,0}
                                  };
        public static readonly int[,] BlockT2 = {
                                     {0,0,3,0},
                                     {0,0,13,8},
                                     {0,0,4,0},
                                     {0,0,0,0}
                                  };
        public static readonly int[,] BlockT3 = {
                                     {0,0,0,0},
                                     {0,7,12,8},
                                     {0,0,4,0},
                                     {0,0,0,0}                             
                                  };
        public static readonly int[,] BlockT4 = {
                                     {0,0,3,0},
                                     {0,7,14,0},
                                     {0,0,4,0},
                                     {0,0,0,0}
                                  };
        // Create matrix of all possible block shapes and their possible rotations
        public static readonly Array blockMatrix = Array.CreateInstance(typeof(int[,]), 7, 4);
        public static readonly Array queuePositionMatrix = Array.CreateInstance( typeof(Vector2), 7);

        
        
        // Create an single block
        public Array blockArray { get; private set; }
        
        
        public int blockRotationIndex { get; private set; }
        public int blockType { get; private set; }
        public EBlockColor blockColor { get; private set; }
        public int posX { get; set; }
        public int posY { get; set; }

        #endregion

        #region Initialization

        public TetrisBlock(EBlockColor color, int type)
            {
            blockType = type;
            blockColor = color;

            blockMatrix.SetValue((int[,])BlockO.Clone(), 0, 0);
            blockMatrix.SetValue((int[,])BlockO.Clone(), 0, 1);
            blockMatrix.SetValue((int[,])BlockO.Clone(), 0, 2);
            blockMatrix.SetValue((int[,])BlockO.Clone(), 0, 3);
            blockMatrix.SetValue((int[,])BlockI.Clone(), 1, 0);
            blockMatrix.SetValue((int[,])BlockI2.Clone(), 1, 1);
            blockMatrix.SetValue((int[,])BlockI.Clone(), 1, 2);
            blockMatrix.SetValue((int[,])BlockI2.Clone(), 1, 3);
            blockMatrix.SetValue((int[,])BlockS.Clone(), 2, 0);
            blockMatrix.SetValue((int[,])BlockS2.Clone(), 2, 1);
            blockMatrix.SetValue((int[,])BlockS.Clone(), 2, 2);
            blockMatrix.SetValue((int[,])BlockS2.Clone(), 2, 3);
            blockMatrix.SetValue((int[,])BlockZ.Clone(), 3, 0);
            blockMatrix.SetValue((int[,])BlockZ2.Clone(), 3, 1);
            blockMatrix.SetValue((int[,])BlockZ.Clone(), 3, 2);
            blockMatrix.SetValue((int[,])BlockZ2.Clone(), 3, 3);
            blockMatrix.SetValue((int[,])BlockL1.Clone(), 4, 0);
            blockMatrix.SetValue((int[,])BlockL2.Clone(), 4, 1);
            blockMatrix.SetValue((int[,])BlockL3.Clone(), 4, 2);
            blockMatrix.SetValue((int[,])BlockL4.Clone(), 4, 3);
            blockMatrix.SetValue((int[,])BlockJ1.Clone(), 5, 0);
            blockMatrix.SetValue((int[,])BlockJ2.Clone(), 5, 1);
            blockMatrix.SetValue((int[,])BlockJ3.Clone(), 5, 2);
            blockMatrix.SetValue((int[,])BlockJ4.Clone(), 5, 3);
            blockMatrix.SetValue((int[,])BlockT1.Clone(), 6, 0);
            blockMatrix.SetValue((int[,])BlockT2.Clone(), 6, 1);
            blockMatrix.SetValue((int[,])BlockT3.Clone(), 6, 2);
            blockMatrix.SetValue((int[,])BlockT4.Clone(), 6, 3);

            blockArray = Array.CreateInstance(typeof(int), 4, 4);
            Array.Copy((int[,])blockMatrix.GetValue(type, 0), blockArray, 16);

           
            queuePositionMatrix.SetValue(new Vector2(0.5f, 0.5f), 0);
            queuePositionMatrix.SetValue(new Vector2(0.52f, 1), 1);
            queuePositionMatrix.SetValue(new Vector2(0, 0.6f), 2);
            queuePositionMatrix.SetValue(new Vector2(0.05f, 0.55f), 3);
            queuePositionMatrix.SetValue(new Vector2(0, 1.5f), 4);
            queuePositionMatrix.SetValue(new Vector2(0, 1.5f), 5);
            queuePositionMatrix.SetValue(new Vector2(0, 1.5f), 6); 


            // This is the initial posistion of the block
            posX = 3;
            posY = -1;
            }

        #endregion

        #region Public Methods

        public bool Rotate(bool rotateRight )
            {
            if (rotateRight)
                {
                blockRotationIndex--;
                if (blockRotationIndex < 0)
                    blockRotationIndex = 3;
                }
            else
                {
                blockRotationIndex++;
                if (blockRotationIndex > 3)
                    blockRotationIndex = 0;
                }

            Array.Copy((int[,])blockMatrix.GetValue(blockType, blockRotationIndex), blockArray, 16);
            
            return true;
            }

        public Texture2D GetTexture()
            {
            return GetTexture(blockColor);   
            }

        public static Texture2D GetTexture(EBlockColor color)
            {
            switch (color)
                {
                    case EBlockColor.eYellow:
                        return GameBoardScreen.textureBrickYellow;
                    case EBlockColor.eCyan:
                        return GameBoardScreen.textureBrickCyan;
                    case EBlockColor.eGreen:
                        return GameBoardScreen.textureBrickGreen;
                    case EBlockColor.eRed:
                        return GameBoardScreen.textureBrickRed;
                    case EBlockColor.eOrange:
                        return GameBoardScreen.textureBrickOrange;
                    case EBlockColor.eBlue:
                        return GameBoardScreen.textureBrickBlue;
                    case EBlockColor.ePurple:
                        return GameBoardScreen.textureBrickPurple;
                }

                // Todo: Add an error texture
                return GameBoardScreen.textureBrickYellow;  
            }

        public Color GetShadowRGB()
            {
            switch(blockColor)
                {
                case EBlockColor.eYellow:
                    return Color.Yellow;
                case EBlockColor.eCyan:
                    return Color.Cyan;
                case EBlockColor.eGreen:
                    return Color.Green;
                case EBlockColor.eRed:
                    return Color.Red;
                case EBlockColor.eOrange:
                    return Color.Orange;
                case EBlockColor.eBlue:
                    return Color.Blue;
                case EBlockColor.ePurple:
                    return Color.Purple;
                }

            return Color.White;
            }


        public Vector2 GetQueuePosition()
            {
            return (Vector2)queuePositionMatrix.GetValue(blockType);
            }
        
        }

        #endregion
}
